@@ -11,6 +11,7 @@
 
 #include "atlas/functionspace/FunctionSpace.h"
 #include "atlas/field/Field.h"
+#include "atlas/grid.h"
 #include "atlas/functionspace/detail/FunctionSpaceImpl.h"
 
 namespace atlas {
@@ -30,6 +31,10 @@ size_t FunctionSpace::footprint() const {
     return get()->footprint();
 }
 
+const Grid& FunctionSpace::grid() const {
+    return get()->grid();
+}
+
 Field FunctionSpace::createField(const eckit::Configuration& config) const {
     return get()->createField(config);
 }
@@ -134,6 +139,11 @@ const Projection& FunctionSpace::projection() const {
     return get()->projection();
 }
 
+const functionspace::HaloDescription& FunctionSpace::halo_description() const {
+    return get()->halo_description();
+}
+
+
 template <typename DATATYPE>
 Field FunctionSpace::createField() const {
     return get()->createField<DATATYPE>();
