@@ -39,6 +39,7 @@ void MeshImpl::encode(eckit::Stream&) const {
 }
 
 MeshImpl::MeshImpl(): nodes_(new mesh::Nodes()), dimensionality_(2) {
+    metadata_.set("mpi_comm",mpi::comm().name());
     createElements();
 }
 
@@ -116,14 +117,27 @@ void MeshImpl::setGrid(const Grid& grid) {
     }
 }
 
-idx_t MeshImpl::nb_partitions() const {
-    return mpi::size();
+idx_t MeshImpl::nb_parts() const {
+    idx_t n;
+    if (not metadata().get("nb_parts", n)) {
+        n = mpi::comm(mpi_comm()).size();
+    }
+    return n;
 }
 
-idx_t MeshImpl::partition() const {
-    return mpi::rank();
+idx_t MeshImpl::part() const {
+    idx_t p;
+    if (not metadata().get("part", p)) {
+        p = mpi::comm(mpi_comm()).rank();
+    }
+    return p;
 }
 
+std::string MeshImpl::mpi_comm() const {
+    return metadata().getString("mpi_comm");
+}
+
+
 void MeshImpl::updateDevice() const {
     if (nodes_) {
         nodes_->updateDevice();
@@ -186,7 +200,7 @@ const PartitionGraph& MeshImpl::partitionGraph() const {
 }
 
 PartitionGraph::Neighbours MeshImpl::nearestNeighbourPartitions() const {
-    return partitionGraph().nearestNeighbours(partition());
+    return partitionGraph().nearestNeighbours(part());
 }
 
 const PartitionPolygon& MeshImpl::polygon(idx_t halo) const {
